import { ref, computed } from '@vue/runtime-core';
import axios from 'axios';

function usePromise(fn, throwException = false) {
    if (!fn) {
        throw new Error(`[usePromise] argument can't be '${fn}'`);
    }
    if (typeof fn !== "function") {
        throw new Error(`[usePromise] expects function, but received ${typeof fn}`);
    }
    const loading = ref(false);
    const error = ref(null);
    const result = ref(null);
    const promise = ref();
    const exec = async (...args) => {
        loading.value = true;
        error.value = null;
        result.value = null;
        const currentPromise = (promise.value = fn(...args));
        try {
            const r = await currentPromise;
            if (promise.value === currentPromise) {
                result.value = r;
            }
            return r;
        }
        catch (er) {
            if (promise.value === currentPromise) {
                error.value = er;
                result.value = null;
            }
            return throwException ? currentPromise : undefined;
        }
        finally {
            if (promise.value === currentPromise) {
                loading.value = false;
            }
        }
    };
    return {
        exec,
        result,
        promise,
        loading,
        error
    };
}

/* istanbul ignore next */
const ExecutionId = Symbol((true !== 'production') ? "RetryId" : undefined);
/* istanbul ignore next */
const CancellationToken = Symbol((true !== 'production') ? "CancellationToken" : undefined);

/* istanbul ignore next  */
const _axios = axios || (globalThis && globalThis.axios);
// TODO replace by shared project
const isObject = (d) => typeof d === 'object';
const isString = (d) => typeof d === 'string';
const isBoolean = (d) => typeof d === 'boolean';
function useAxios(configUrl, configThrowException, throwException = false) {
    /* istanbul ignore next  */
    (true !== 'production') &&
        !_axios &&
        console.warn(`[axios] not installed, please install it`);
    const config = !isString(configUrl) ? configUrl : isObject(configThrowException) ? configThrowException : undefined;
    throwException = isBoolean(configThrowException) ? configThrowException : throwException;
    const axiosClient = _axios.create(config);
    const client = computed(() => axiosClient);
    const use = usePromise(async (request) => {
        return axiosClient.request(request);
    }, throwException);
    const data = computed(() => (use.result.value && use.result.value.data) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.data) ||
        null);
    const status = computed(() => (use.result.value && use.result.value.status) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.status) ||
        null);
    const statusText = computed(() => (use.result.value && use.result.value.statusText) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.statusText) ||
        null);
    // if url provided in the config, execute it straight away
    if (typeof configUrl === 'string') {
        use.exec({ url: configUrl });
    }
    else if (config && config.url) {
        use.exec(config);
    }
    return {
        ...use,
        client,
        data,
        status,
        statusText
    };
}

export { useAxios };
